/*
 * Copyright (c) 2009 Shanti Subramanyam, Akara Sucharitakul

 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package com.testnscale.sampleweb;

import com.sun.faban.common.NameValuePair;
import com.sun.faban.common.Utilities;
import com.sun.faban.driver.*;
import com.sun.faban.driver.util.ContentSizeStats;

import javax.xml.xpath.XPathExpressionException;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Basic web workload driver drives only one operation via a get request.
 */
@BenchmarkDefinition (
    name    = "SampleWeb Workload",
    version = "0.1",
    metric  = "req/s"

)
@BenchmarkDriver (
    name             = "SampleWeb",
    threadPerScale   = 1,
    opsUnit          = "requests",
    metric           = "req/s",
    responseTimeUnit = TimeUnit.MILLISECONDS

)
@FixedTime (
    cycleType = CycleType.THINKTIME,
    cycleTime = 0,
    cycleDeviation = 5
)
@MatrixMix (
    operations = {"Home", "Products", "Contacts"},
    mix = { @Row ({ 0, 80, 20 }),
            @Row ({20, 39, 41 }),
            @Row ({60, 19, 21 })
          }
)
public class SampleWebDriver {

    private DriverContext ctx;
    private HttpTransport http;
    private String url, homeUrl, productsUrl, contactsUrl;
    ContentSizeStats contentStats = null;

    /**
     * Constructs the basic web workload driver.
     * @throws XPathExpressionException An XPath error occurred
     */
    public SampleWebDriver() throws XPathExpressionException, ConfigurationException {
        ctx = DriverContext.getContext();
        HttpTransport.setProvider(
                "com.sun.faban.driver.transport.hc3.ApacheHC3Transport");
        http = HttpTransport.newInstance();
        StringBuilder urlBuilder = new StringBuilder();

        String s = ctx.getProperty("secure");
        if ("true".equalsIgnoreCase(s))
            urlBuilder.append("https://");
        else
            urlBuilder.append("http://");

        s = ctx.getXPathValue("/sampleWeb/webServer/fa:hostConfig/fa:hostPorts");
        List<NameValuePair<Integer>> hostPorts =
                Utilities.parseHostPorts(s);
        // We currently only support a single host/port with this workload
        if (hostPorts.size() != 1)
            throw new ConfigurationException(
                    "Only single host:port currently supported.");
        NameValuePair<Integer> hostPort = hostPorts.get(0);
        urlBuilder.append(hostPort.name);
        if (hostPort.value != null)
            urlBuilder.append(':').append(hostPort.value);
        
        url = urlBuilder.toString();

        s = ctx.getProperty("homePath");
        if (s.charAt(0) == '/')
            homeUrl = url + s;
        else
            homeUrl = url + '/' + s;
        s = ctx.getProperty("productsPath");
        if (s.charAt(0) == '/')
            productsUrl = url + s;
        else
            productsUrl = url + '/' + s;
        s = ctx.getProperty("contactsPath");
        if (s.charAt(0) == '/')
            contactsUrl = url + s;
        else
            contactsUrl = url + '/' + s;
        contentStats = new ContentSizeStats(ctx.getOperationCount());
        ctx.attachMetrics(contentStats);
    }

    /**
     * Do the request.
     * @throws IOException An I/O or network error occurred.
     */
    @BenchmarkOperation (
        name    = "HomePage",
        max90th = 250, // 250 millisec
        timing  = Timing.AUTO
    )
    public void doHomePage() throws IOException {
        int size = http.readURL(homeUrl);
        if (ctx.isTxSteadyState())
            contentStats.sumContentSize[ctx.getOperationId()] += size;
    }

    @BenchmarkOperation (
        name    = "ProductsPage",
        max90th = 250, // 250 millisec
        timing  = Timing.AUTO
    )
    public void doProductsPage() throws IOException {
        int size = http.readURL(productsUrl);
        if (ctx.isTxSteadyState())
            contentStats.sumContentSize[ctx.getOperationId()] += size;
    }

    @BenchmarkOperation (
        name    = "ContactsPage",
        max90th = 250, // 250 millisec
        timing  = Timing.AUTO
    )
    public void doContactsPage() throws IOException {
        int size = http.readURL(contactsUrl);
        if (ctx.isTxSteadyState())
            contentStats.sumContentSize[ctx.getOperationId()] += size;
    }
}
